@@ -29,11 +29,6 @@
  * Any downstream error received while the #GstValve:drop property is %FALSE
  * is ignored. So downstream element can be set to  %GST_STATE_NULL and removed,
  * without using pad blocking.
- *
- * This element was previously part of gst-plugins-farsight, and then
- * gst-plugins-bad.
- *
- * Documentation last reviewed on 2010-12-30 (0.10.31)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -120,6 +115,7 @@ gst_valve_init (GstValve * valve)
       GST_DEBUG_FUNCPTR (gst_valve_event));
   gst_pad_set_query_function (valve->srcpad,
       GST_DEBUG_FUNCPTR (gst_valve_query));
+  GST_PAD_SET_PROXY_CAPS (valve->srcpad);
   gst_element_add_pad (GST_ELEMENT (valve), valve->srcpad);
 
   valve->sinkpad = gst_pad_new_from_static_template (&sinktemplate, "sink");
@@ -130,6 +126,7 @@ gst_valve_init (GstValve * valve)
   gst_pad_set_query_function (valve->sinkpad,
       GST_DEBUG_FUNCPTR (gst_valve_query));
   GST_PAD_SET_PROXY_CAPS (valve->sinkpad);
+  GST_PAD_SET_PROXY_ALLOCATION (valve->sinkpad);
   gst_element_add_pad (GST_ELEMENT (valve), valve->sinkpad);
 }
 
